%% roughness parameter calculation (wo and lambda) from a .xyz file
%(raw coordinates).  Original script by W.A. Griffith, modified 
% F. Lazari and S. Nielsen.

%3 loading modes: -single file: use rows 17:20 to test script. Remove for cycles, see comments.
%                 -natural samples: two for loops to automatically analyze
%                 the whole natural dataset. Uncomment 23:26 for sample
%                 name list and 38:56 for data loading
%                 -experimental samples: as for natural samples. uncomment
%                 29:31, 41, 48, 52.
%data definition and loading sections are messy but time-saving when analysing whole dataset

clear all
clc

ext='.xyz'; %file extension
row=1; %first row for excel data writing

%% 1 data loading

% single file loading: use to test script
fname='L05-02';
or=''; %N, S, W or E vein sides
path=['D:\tesi magistrale\profiles\', fname]; %uncomment for single file loading
data = load([path,'\',fname,or,ext]); %load single .xyz profile
scale=15.59;

% %%%%%%%%%%%%% Natural dataset loading%%%%%%%%%%%%%
% fname=["A1" "A02-02", "A02-04","A2", "A4", "A5", "A7", "L05-02", "L05-03", "L05-04",...
%     "L05-06", "L05-07", "L05-08", "L09-01a", "L09-01b", "L09-02a",...
%     "L09-02b", "L09-03a", "L09-03b", "W05-S01", "W05-S04", "W09-S01"];
% scale=[20.04, 19.06, 16.51, 16.58, 34.20, 32.21, 34.52, 32.55, 33.67, 33.26, ...
%     32.5, 31.27, 32.7, 30.78, 15.59, 19.01, 17.21, 16.95, 19.57, 19.6, 16.91, ...
%     6.68, 18.96, 19.91, 32.86, 32.87, 33.92, 33.31, 23.84, 18.24, 15.48, ...
%     11.12, 32.52, 32.37, 16.38, 11.15, 27.07, 23.63, 31.57, 17.51, 30.27, 31.88];
%
% % fname=["A1", "A02-02", "A5","A7", "W05-S01", "W09-S01"];%file name of redrawn profiles
% % scale=[14.64, 7.56, 32.5, 21.55, 29.38, 23.63, 22.64]; %length of redrawn profiles
% % o=1;
% % f=6;
% count=0;
% for f=1:length(fname) %for loop for every sample, comment for single file loading (remember end)
%     path=append('D:\tesi magistrale\profiles\', fname(f)); %path for multiple file loading
% %        natural samples orientation ALL PROFILES
%     if f==1||f==4||f==20  %injection veins orientation
%         or=["E", "W"];
%     elseif f==8 || f==9
%         or=["O"];
%     else
%         or=["N","S"]; %fault veins orientation
%     end
%
% %    natural samples orientation REDRAWN PROFILES
% %     if f==1 %injection veins orientation
% %         or=["E"];
% %     elseif f==5
% %         or=["W"];
% %     elseif f==3
% %         or=["N","S"]; %fault veins orientation
% %     elseif f==6
% %         or=["N"];
% %     else
% %         or=["S"];
% %     end
%     for o=1:length(or) %for loop for every side of the sample (remember end)
%         count=count+1;
%         if f==8||f==9 %single-side sections
%             data = load(append(path,'\',fname(f),ext)); %import .xyz profile for single side sections
%         else
%             data = load(append(path,'\',fname(f),or(o),ext)); %import multiple .xyz profile
%         end
%         %%%%%%%%%%%%%%%%%%%% experimental dataset loading %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%         count=0;
%         
%         fname=["HRV376","S422","S423","S475"];
%         scale=[20.32, 18.73, 9.81, 16.21, 10.14, 16.28, 4.08, 6.93, 4.00, 6.93, 5.89, 12.15, 5.61, 11.97];
%         f=4;
%         o=1;
%         count=11;
%         row=count+1;
% %         for f=1:length(fname)
%             path=append('D:\tesi magistrale\profiles\', fname(f)); %path for multiple file loading
%             if f ==1
%                 or=["semtop","sembot"];
%             else
%                 or=["semtop","sembot","omtop","ombot"];
%             end
% %             for o=1:length(or)
% %                 count=count+1
%                 data = load(append(path,'\',fname(f),or(o),ext));
%                 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



x = data(:,1)-min(data(:,1)); %translate x-origin to x=0
y = data(:,2)-mean(data(:,2));%translate y-origin to mean y
scalefactor = scale(count)/max(x);
x = scalefactor*x;y = scalefactor*y;%apply scale factor to both x and y axis

%rotate profile
num = length(x);
rot = atan((y(1)-y(num))/(x(1)-x(num)));%figure out rotation angle by finding slope of overall trend line
x = x*cos(rot)+y*sin(rot); y = -x*sin(rot)+y*cos(rot);%rotate so that fault tips end at the same f
x = x-min(x); %translate x-origin to x=0
y = y-mean(y);%translate y-origin to mean y

%plot initial fracture profile
figure(1)
clf
plot(x,y)
xlabel('x (mm)')
ylabel('y (mm)')
title('fracture profile')
axis equal

%% 3 profile elaboration

%resort data so that datapoints are increasing in x (not necessary)
XY = sortrows([x y]);
x1 = XY(:,1); y1 = XY(:,2);

%differentiate points with the same x (made during drawing)
for m = 1:length(x1)-1
    if x1(m)==x1(m+1)
        x1(m+1)=x1(m)+0.1*mean(diff(x1));
    end
end

% plot data after sorting
figure(2)
clf
plot(x1,y1)
xlabel('x (mm)')
ylabel('y (mm)')
title('fracture profile after sorting')
%plot reference line and overall profile trend
X = [ones(length(x1),1) x1];
m = lscov(X,y1); %least square with known covariance, find ovewrall slope
yt=m(2).*x1+m(1); %overall slope
ref =mean(y1); %average y value
rl=ref+zeros(length(x1)); %vector for reference line plotting
hold on
plot(x1, rl)
plot (x1, yt)
legend('fracture profile', 'reference line', 'overall trend')
axis equal
hold off

%calculate deviation of profile from reference line
z_rough=y1-ref;

figure (3)
clf
plot(x1,z_rough)
xlabel('x (mm)')
ylabel('z (mm)')
title([fname, or, ' Deviation from Reference Line'])
title(append(fname(f), or(o), ' selected, Deviation from Reference Line'))%use this with multiple samples
axis equal
exportgraphics(gcf,append(path,'/',fname (f),or (o), 'new profile.png'), 'resolution', 300) %save to sample folder
%         %calculate wo from height distribution
[H1, X] = hist(z_rough, 20);
wo = sqrt(1/length(z_rough)*sum(z_rough.^2));
%bar graph representing asperities height distribution
figure (4)
barh(X, H1)
ylabel('z (mm)')
xlabel('N')
%         title(append(fname, or,' \omega_o = ', num2str(wo), ' +/-', num2str(std(z_rough))));
title(append(fname(f), or(o),' selected: \omega_o = ', num2str(wo), ' +/-',num2str(std(z_rough)))); %use with multiple samples

%% 4 profile filtering and interpolation

%FILTERING CAUSES PROBLEMS, LEAVE COMMENTED 138:145 WHEN CALCULATING DEFINITIVE DATA
%moving average to smooth small variations

%for lm=3:7 %uncomment to test different moving window sizes (uncomment end)

% lm = 4; %characteristic width of moving window
% a=1; %a=1, characteristic height of moving window.
% b = (1/lm)*ones(1, lm);
% z = filter(b,a,z_rough); %unidimensional filter, moving window is b*a

z1=z_rough;

%interpolate and resample using linear interpolation to get uniformly spaced dataset
x2 = min(x1):mean(diff(x1)):max(x1); %build uniformly-spaced x-axis
z2 = interp1(x1, z1, x2, 'linear');
x = x2; z = z2;

figure(5)
clf
subplot (1,2,1)
plot(x1, z1)
xlabel('x (mm)')
ylabel('z (mm)')
title(append(fname,or,' fracture profile after linear interpolation'))
title(append(fname(f),or(o),'selected: fracture profile after linear interpolation')) %use with multiple samples

hold on
plot(x,z, 'k:')
legend('before interpolation', 'after interpolation')
axis equal
subplot (1,2,2)
barh(X, H1)
ylabel('z (mm)')
xlabel('N')
title(append(fname, or,' \omega_o = ', num2str(wo), ' +/-',num2str(std(z_rough)))); %use with multiple samples
title(append(fname (f), or (o),'selected \omega_o = ', num2str(wo), ' +/-',num2str(std(z_rough)))); %use with multiple samples
hold off

%% 5 curvature parameters

%call function lmin.m to find local minima (valleys), there is also lmax (same results? I did not test it yet)
[lmval,i]=lmin(z,5); %lmval is the local minima value, i is the x position
%         scatter(x(i),z(i)) %check where minima are
%         legend('before interpolation', 'after interpolation','minima')
%         hold off
dcX=zeros (1, length(x)-1);
dcZ=dcX;

%loop to calculate x and z shift between j-1 and j+1
for j = 2:length(x)-1
    dcX(j) = x(j+1)-x(j-1);
    dcZ(j) = z(j+1)-z(j-1);
end

dcX(1)=mean([dcX(2) dcX(end)]);
dcZ(1) = mean([dcZ(2) dcZ(end)]);

ds = sqrt(dcX.^2+dcZ.^2); %arc length between each pair of adjacent points
tX = dcX./ds;%x-component of tangent vector between two points
tZ = dcZ./ds;%Z-component of tangent vector between two points

kZ=zeros (1, length(x)-1);
kX=zeros (1, length(x)-1);

for j = 2:length(x)-2
    kX(j) = (tX(j+1)-tX(j-1))./ds(j);%x-component of curvature vector (2nd space derivative in x direction)
    kZ(j) = (tZ(j+1)-tZ(j-1))./ds(j);%z-component of curvature vector (2nd space derivative in z direction)
end
kX(1)=1;
kZ(1)=1;


% curvature parameter 1: curvature radius (R)

kappa = sqrt(kX(i).^2+kZ(i).^2); %scalar curvature calculated at peaks
R = 1./kappa;%radius of curvature calculated at peaks

figure (6)
subplot 131
hist(R)
xlabel('R (mm)')
ylabel('N')
title(['R_{ave}= ' num2str(mean(R))]);

%% 6 Spectral analysis

% input parameters
dx = mean (diff(x)); % sampling step
fs  = 1/dx; % spatial sampling frequency
np = length (x); %number of points in profile (length of X)
Lx = (0:np-1)*dx;
% Fourier transform and spectrum plot
yt = fft(z);
P2 = abs(yt/np);
P1 = P2(1:np/2+1);
P1(2:end-1) = 2*P1(2:end-1);
ft = fs*(0:(np/2))/np;
wl=1./ft;
F = polyfit(log(wl(2:end)),log(P1(2:end)),1);

figure(9)
clf
plot(wl,P1)
xlabel ('Wavelength (mm)')
xlim([8*10^-3 40])
ylabel('Magnitude')
ylim([10^-7 2])
title (append(fname(f),' ', or(o), ' selected Fourier transform, slope=', num2str(F(1))))
title (append(fname,' ', or, ' Fourier transform spectral analysis, slope=', num2str(F(1))))

Write data to text file
T = [wl ;P1];
size(T)

%% 7 wo1 (root mean square of asperities height)
z_s = z(i); %find peaks elevation
[H1, X] = hist(z_s, 20);
wo1 = sqrt(1/length(z_s)*sum(z_s.^2));  %root mean square

figure (7)
clf
subplot 211
bar(X, H1,1)
xlabel('z (mm)'), ylabel('N')
title([fname, or, ' \omega_o = ' num2str(wo1) 'mm, +/-' num2str(std(z_s))]) %single sample
title(append(fname(f), ' ', or(o),' selected \omega_o = ', num2str(wo), ' +/-',num2str(std(z_rough))));  %multiple samples
axis square


%% 8 lambda_ave
%approximate wavelength
r = 1;
while r <= length(R)
    if R(r) > 1
        R(r) = [];
    end
    r = r+1;
end

lambda = R;
[H2, X] = hist(lambda, 20);

subplot 212
bar(X, H2, 1)
xlabel('\lambda (mm)')
ylabel('N')
colormap('white')
axis square
title([fname, or ', \lambda_{ave}= ' num2str(mean(lambda)) 'mm, +/-' num2str(std(lambda))]); %single sample
title(append(fname(f), ' ', or(o),' selected \lambda_{ave} = ', num2str(mean(lambda)), ' +/-',num2str(std(lambda))));  %multiple samples

%% 9 producing a global view of the parameters

figure ('WindowState', 'maximize')
clf

subplot (2, 3, [1, 4]) %thin section image
image= imread(append ('D:\tesi magistrale\thesis\', fname (f), '.png'));
imshow (image(:,:,1:3))

subplot 232 %profile
plot(x,z)
axis equal
xlabel('(mm)')
ylabel('(mm)')
xlim([0 max(scale)])
ylim([-2 2])
set(gca, 'FontName', 'Times New Roman', 'FontSize', 12)
title (append('Fault ',or(o),' profile'))

subplot 233 %wo related to the profile
[H1, X] = hist(z, 20);
wo = sqrt(1/length(z)*sum(z.^2));
barh(X, H1)
ylabel('z (mm)')
ylim([-2 2])
xlabel('N')
set(gca, 'FontName', 'Times New Roman', 'FontSize', 12)
title(append('\omega_o = ', num2str(wo), ' +/-',num2str(std(z_rough))));

subplot (2, 3, 5)
loglog(wl,P1)
xlabel ('Wavelength (mm)')
xlim([8*10^-3 40])
ylabel('Magnitude')
ylim([10^-7 2])
%         title (append(fname(f),' ', or(o), ' Fourier transform, slope=', num2str(F(1))))
title (append(fname,' ', or, ' Fourier transform, slope=', num2str(F(1))))

subplot 236
[H2, X] = hist(lambda, 20);
bar(X, H2, 1)
xlabel('\lambda (mm)')
ylabel('N')
colormap('white')
axis square
set(gca, 'FontName', 'Times New Roman', 'FontSize', 12)
title(append('\lambda_{ave} = ', num2str(mean(lambda)), ' +/-',num2str(std(lambda))));

sgtitle(append(fname(f), ' ', or(o)))
set(gca, 'FontName', 'Times New Roman', 'FontSize', 12)

%global view without lambda

figure ('WindowState', 'maximize')
clf

subplot (2, 2, 1) %thin section image
image= imread(append ('D:\tesi magistrale\thesis\', fname (f), '.png'));
imshow (image(:,:,1:3))

subplot 222 %profile
plot(x,z)
axis equal
xlabel('(mm)')
ylabel('(mm)')
xlim([0 max(scale)])
ylim([-2 2])
set(gca, 'FontName', 'Times New Roman', 'FontSize', 12)
title (append('Fault ',or(o),' profile'))

subplot 223 %wo related to the profile
[H1, X] = hist(z, 20);
wo = sqrt(1/length(z)*sum(z.^2));
barh(X, H1)
ylabel('z (mm)')
ylim([-2 2])
xlabel('N')
set(gca, 'FontName', 'Times New Roman', 'FontSize', 12)
title(append('\omega_o = ', num2str(wo), ' +/-',num2str(std(z_rough))));

subplot 224 %fourier transform
loglog(wl,P1)
xlabel ('Wavelength (mm)')
xlim([8*10^-3 40])
ylabel('Magnitude')
ylim([10^-7 2])
set(gca, 'FontName', 'Times New Roman', 'FontSize', 12)
title (append(fname(f),' ', or(o), ' Fourier transform, slope=', num2str(F(1))))

sgtitle(append(fname(f), ' ', or(o)))
set(gca, 'FontName', 'Times New Roman', 'FontSize', 12)


%% 10 writing data to excel worksheet

xlsname='Roughness data.xls'; %excel filename

xlswrite(['D:\tesi magistrale\', xlsname], cellstr(append(fname(f),or(o))),1,['A',num2str(row)]); %sample name and orientation
xlswrite(['D:\tesi magistrale\', xlsname], mean(lambda),1,['B',num2str(row)]); %lambda
xlswrite(['D:\tesi magistrale\', xlsname], std(lambda),1,['C',num2str(row)]); %lambda standard deviation
xlswrite(['D:\tesi magistrale\', xlsname], wo,1,['D',num2str(row)]); %w0 calculated on distribution
xlswrite(['D:\tesi magistrale\', xlsname], std(z_rough),1,['E',num2str(row)]); %wo standard deviation
xlswrite(['D:\tesi magistrale\', xlsname], wo1,1,['F',num2str(row)]); %w0 calculated on asperities
xlswrite(['D:\tesi magistrale\', xlsname], std(z_s),1,['G',num2str(row)]); %wo standard deviation
xlswrite(['D:\tesi magistrale\', xlsname], F(1),1,['J',num2str(row)]); %fourier spectrum overall slope
xlswrite(['D:\tesi magistrale\', xlsname], F(2),1,['K',num2str(row)]); %fourier spectrum intercept

row=row+1; %update writing row



%         end %uncomment to test different filter window sizes
%     end %uncomment both end for multiple file loading
% end
disp('done');

